#ifndef _ARCLIB_H
#define	_ARCLIB_H

#include "commdef.h"
#include "interface.h"
#ifdef _WIN32
#include <Winsock2.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

//	THE BELOW BLOCK IS ONLY FOR INTERNAL USE
/************************************************************************************/
enum
{
GUI_SET_SERIAL = 0x10,
GUI_SET_VENDOR,
GUI_SET_MODEL,
GUI_IDENTIFY,
GUI_CHECK_PASSWORD,
GUI_LOGOUT,
GUI_HTTP,
GUI_SET_ETHERNET_ADDR,
GUI_SET_LOGO,
GUI_POLL_EVENT,
GUI_GET_EVENT,
GUI_GET_HW_MONITOR,
GUI_SMART_TEST,
GUI_SET_TIME,
GUI_SNMP,

GUI_GET_INFO_R = 0x20,
GUI_GET_INFO_V,
GUI_GET_INFO_P,
GUI_GET_INFO_S,
GUI_CLEAR_EVENT,

GUI_MUTE_BEEPER = 0x30,
GUI_BEEPER_SETTING,
GUI_SET_PASSWORD,
GUI_HOST_INTERFACE_MODE,
GUI_REBUILD_PRIORITY,
GUI_MAX_ATA_MODE,
GUI_RESET_CONTROLLER,
GUI_COM_PORT_SETTING,
GUI_NO_OPERATION,
GUI_DHCP_IP,

GUI_CREATE_PASS_THROUGH = 0x40,
GUI_MODIFY_PASS_THROUGH,
GUI_DELETE_PASS_THROUGH,
GUI_IDENTIFY_DEVICE,

GUI_CREATE_RAIDSET = 0x50,
GUI_DELETE_RAIDSET,
GUI_EXPAND_RAIDSET,
GUI_ACTIVATE_RAIDSET,
GUI_CREATE_HOT_SPARE,
GUI_DELETE_HOT_SPARE,

GUI_CREATE_VOLUME = 0x60,
GUI_MODIFY_VOLUME,
GUI_DELETE_VOLUME,
GUI_START_CHECK_VOLUME,
GUI_STOP_CHECK_VOLUME
};

// Application specific
enum 
{
GUI_GET_TCPPORT = 0xF0,
GUI_GET_COMATTR,

// status code
GUI_WOULDBLOCK,
GUI_TIMEOUT
};
/****************************************************************************************/


//
// Library returned status
//
#define ARC_STATUS					int
#define ARC_SUCCESS					0x00
#define ARC_FAILURE					0x01	//something error but not defined
#define ARC_RETURN_FRAME_ERROR 		0x02	//The data returned from rs-232 error
#define ARC_INVALID_COM_HANDLE		0x03	//invalid Rs-232 handle
#define ARC_DATA_TRANSMIT_ERROR		0x04	//failed to write to media
#define ARC_DATA_RECEIVE_ERROR		0x05	//failed to read from media( no response from raid subsystem)
#define ARC_PARAMETER_ERROR			0x06	//the wrong parameter(s)
#define ARC_OPENFILE_ERROR			0x07	//failed to open file
#define ARC_PASSWORD_REQUIRED       0x08	//password required
#define ARC_INVALID_PASSWORD        0x09	//wrong password
#define ARC_INSUFFICIENT_MEM		0x0A	//memory too small
#define ARC_CANT_ALLOC_MEM			0x0B	//failed to allocate necessary memory
#define ARC_BUFFER_TOO_SMALL		0x0C	//The passed buffer is too small to put the returned data in
#define	ARC_SOCKET_ERROR			0x0D	//
#define ARC_INVALID_SOCKET			0x0E	//
#define ARC_INIT_COM_ERROR			0x0F	// failed to open COM port
#define ARC_SOCKET_STARTUP_ERROR	0x10	// failed to init socket
#define ARC_INVALID_INTERFACE		0x11
#define ARC_NO_SUCH_DEVICE			0x12	// device not present( including raidset, volumeset, physical drive )

// RAID controller returned status
#define GUI_OK                    0x41
#define GUI_RAIDSET_NOT_NORMAL    0x42
#define GUI_VOLUMESET_NOT_NORMAL  0x43
#define GUI_NO_RAIDSET            0x44
#define GUI_NO_VOLUMESET          0x45
#define GUI_NO_PHYSICAL_DRIVE     0x46
#define GUI_PARAMETER_ERROR       0x47
#define GUI_UNSUPPORTED_COMMAND   0x48
#define GUI_DISK_CONFIG_CHANGED   0x49
#define GUI_INVALID_PASSWORD      0x4a
#define GUI_NO_DISK_SPACE         0x4b
#define GUI_CHECKSUM_ERROR        0x4c
#define GUI_PASSWORD_REQUIRED     0x4d

// flags of getting event
#define FLAG_ALL			0
#define FLAG_NEW			1

// Beeper settings
#define BEEPER_ON			1
#define BEEPER_OFF			0

//
#define RAID				0
#define JBOD				1


#define DISABLE				0
#define ENABLE				1


#define	TYPE_64BIT_LBA			0
#define	TYPE_4K_FOR_WIN			1


// host type interface
// HostType 
#define HOST_PCI			1
#define HOST_PCIExpress		2
#define HOST_PATA			3
#define HOST_SATA1			4
#define HOST_SATA2			5
#define HOST_SCSI160		6
#define HOST_SCSI320		7
#define HOST_Fibre2G		8
#define HOST_Fibre4G		9

// TargetType

#define TARGET_PATA			1
#define TARGET_SATA			2
#define TARGET_SCSI			3
#define TARGET_Fibre		4


//=======================================================
// Definitions of raidset status
//=======================================================
#define  RAID_NORMAL   		0x01  /* Raid set in use */
#define  RAID_COMPLETE  	0x04  /* 0 if incomplete */
#define  RAID_PASSTHROUGH 	0x80  /* Pass through raid set */


//=======================================================
// Definitions of volume status
// Array status == OR of all of its volumes
// volStatus
//=======================================================
#define  VOLUME_NORMAL				0x00
#define  VOLUME_INITIALIZING		0x01
#define  VOLUME_FAILED				0x02
#define  VOLUME_MIGRATING			0x04
#define  VOLUME_REBUILDING			0x08
#define  VOLUME_NEED_INIT			0x10
#define  VOLUME_NEED_MIGRATE		0x20
#define  VOLUME_INIT_FLAG			0x40
#define  VOLUME_NEED_REGEN			0x80 
#define  VOLUME_CHECKING			0x100
#define  VOLUME_NEED_CHECK			0x200


//=======================================================
// Definitions of volume type
//=======================================================
#define  VOLUME_TYPE_STRIPING 		0  // Striping
#define  VOLUME_TYPE_MIRROR  		1  // Mirror 
#define  VOLUME_TYPE_RAID3  		2  // RAID 3 volume
#define  VOLUME_TYPE_RAID5  		3  // RAID 5
#define  VOLUME_TYPE_RAID6  		4  // RAID 6
#define  VOLUME_TYPE_PASSTHROUGH 	5  // Pass through volume

//=======================================================
// Definitions of volume stripe size
//=======================================================
#define STRIPE_SIZE_4K		0
#define STRIPE_SIZE_8K		1
#define STRIPE_SIZE_16K		2
#define STRIPE_SIZE_32K		3
#define STRIPE_SIZE_64K		4
#define STRIPE_SIZE_128K	5


#define SCSI_SPEED_ASYNC	0
#define SCSI_SPEED_20		1
#define SCSI_SPEED_40		2
#define SCSI_SPEED_80		3
#define SCSI_SPEED_160		4

#define IDE_SPEED_33		0
#define IDE_SPEED_66		1
#define IDE_SPEED_100		2
#define IDE_SPEED_133		3
#define IDE_SPEED_150		4

#define BACKGROUND_INIT		0
#define FOREGROUND_INIT		1

// COM port attributes
typedef struct sCOM_ATTR
{
	BYTE comBaudRate;				// BaudRate: 0/1/2/3/4/5/6/7 (1200/2400/4800/9600/19200/38400/57600/115200)					
	BYTE comDataBits;				// DataBit: always 1 (8 bits)
	BYTE comStopBits;				// StopBit: (0:1, 1:2 stop bits)
	BYTE comParity;					// Parity: always 0 (no parity)
	BYTE comFlowControl;			// FlowControl: always 0 (no flow control)
} sCOM_ATTR, *pCOM_ATTR;

// SCSI attribtes
typedef struct sSCSI_ATTR
{
	BYTE scsiChannel;				// channel for SCSI target (0/1)
	BYTE scsiId;					// SCSI ID for the volume
	BYTE scsiLun;					// LUN (0 if target is IDE)
	BYTE scsiTaggedQueuing;			// 1 --> queue enabled
	BYTE scsiCacheMode;				// 1 --> Cache enabled
	BYTE scsiSpeed;					// IDE(0:ATA33/1:ATA66/2:ATA100/3:ATA133)
} sSCSI_ATTR, *pSCSI_ATTR;


// Rebuild priority
enum
{
	REBUILD_PRIORITY_ULOW,        // 5%
	REBUILD_PRIORITY_LOW,         // 20%
	REBUILD_PRIORITY_MEDIUM,      // 50%
	REBUILD_PRIORITY_HIGH         // 80%
};

typedef struct sSYSTEM_INFO
{
	BYTE gsiVendorName[40];			// Vendor name
	BYTE gsiSerialNumber[16];		// Serial number
	BYTE gsiFirmVersion[16];		// Version of firmware 
	BYTE gsiBootVersion[16];		// Version of boot
	BYTE gsiMbVersion[16];			// Version of MBR(Master Boot Record)
	BYTE gsiModelName[8];			// Model name
	BYTE gsiLocalIp[4];				// Static IP if DHCP disabled
	BYTE gsiCurrentIp[4];			// Currently IP Address, the value is either assigned by DHCP server if DHCP enabled or static IP if DHCP disabled.
	DWORD gsiTimeTick;				// Raid subsystem currently time tick
	DWORD gsiCpuSpeed;				// CPU speed ( in MHz )
	DWORD gsiICache;				// CPU internal Instruction cache size ( in bytes )
	DWORD gsiDCache;				// CPU internal Data Cache size	( in bytes )
	DWORD gsiScache;				// CPU secondary cache size	( in bytes )
	DWORD gsiMemorySize;			// Memory size ( in MBs )
	DWORD gsiMemorySpeed;			// Memory speed ( in MHz )
	DWORD gsiEvents;				// # of IDE events supported
	BYTE gsiMacAddress[6];			// MAC address if ethernet support
	BYTE gsiDhcp;					// 0: DHCP function disabled, 1: DHCP function enabled
	BYTE gsiBeeper;					// 0: Beeper disabled, 1: Beeper enabled
	BYTE gsiChannelUsage;			// 
	BYTE gsiMaxAtaMode;				// 
	BYTE gsiSdramEcc; 				// 1:if ECC enabled
	BYTE gsiRebuildPriority;		// Initialize/rebuild/migration priority
	sCOM_ATTR gsiComA; 				// Attribute of COM1
	sCOM_ATTR gsiComB; 				// Attribute of COM2
	BYTE gsiIdeChannels;			// # of IDE channels supported
	BYTE gsiScsiHostChannels;		// # of SCSI host channels supported
	BYTE gsiIdeHostChannels;		// # of IDE host channels supported
	BYTE gsiMaxVolumeSet;			// # of VolumeSets supported
	BYTE gsiMaxRaidSet;				// # of RaidSet supported
	BYTE gsiEtherPort; 				// 1:If ethernet port supported
	BYTE gsiRaid6Engine; 			// 1:Raid6 engine supported
	BYTE gsiRs232Snmp;				// 1:SNMP through RS232
	BYTE gsiMIB_1;
	BYTE gsiMIB_2;
	DWORD gsiEnterpriseNum;			// Enterprise number of SNMP
    BYTE gsiHostType;               // Host type
    BYTE gsiTargetType;             // Target type
    BYTE gsiInBandEnable;           // 1: if Inband enabled
    BYTE gsiInBandChannel;          // The SCSI channel of inband device
    BYTE gsiInBandId;               // The SCSI ID of inband device
    BYTE gsiRes[63];                // 
} sSYSTEM_INFO, *pSYSTEM_INFO;

typedef struct sGUI_RAIDSET
{
	BYTE grsRaidSetName[16];	// 16 bytes raidset name
	DWORD grsCapacity;			// Minimum capacity in blocks required for each disk (member disk's minimum capacity)
	DWORD grsCapacityX;			// reserved for 64 bit extension
	DWORD grsFailMask;			// bit mask to indicate the failed drive (bit0 -- drive 0, bit1 -- drive 1 ....)
	BYTE grsDevArray[32];		// member disk's channel # ( start from 0 )
	BYTE grsMemberDevices;		// Number of member disks in this raid set
	BYTE grsNewMemberDevices;	// New number of disks in this raid set (in case of raid EXPANDing)
	BYTE grsRaidState;			// Current state of RAID set
	BYTE grsVolumes;			// # of volumes created in this raid set
	BYTE grsVolumeList[16];		// array of VOLUME NUMBER of the created volumes in this raid set
	BYTE grsRes1;				// reserved
	BYTE grsRes2;				// reserved
	BYTE grsRes3;				// reserved
	BYTE grsFreeSegments;		// # of free segment in this raid set
	DWORD grsRawStripes[8];		// Free segment array, each segment is 0x200 blocks for each member disk
	DWORD grsRes4;				// reserved
	DWORD grsRes5;				// reserved
	DWORD grsRes6;				// reserved
} sGUI_RAIDSET, *pGUI_RAIDSET;

typedef struct sGUI_VOLUMESET
{
	BYTE gvsVolumeName[16];		// 16 bytes volumeset name
	DWORD gvsCapacity;			// Volume Capacity (in blocks)
	DWORD gvsCapacityX;			// Reserved for 64 bits
	DWORD gvsFailMask;			// Failed mask for the volume (same as raidset)
	DWORD gvsStripeSize;		// (value in blocks) 8--4K, 16--8K, 32--16K, 64--32K, 128--64K, 256--128K
	DWORD gvsNewFailMask;		// For internal use only
	DWORD gvsNewStripeSize;		// New stripe size ( used for MIGRATION)
	DWORD gvsVolumeStatus;		// see above ( volume status )
	DWORD gvsProgress; 			// Progress for the volume if in (initializing/rebuilding/migrating)
	sSCSI_ATTR gvsScsi; 		
	BYTE gvsMemberDisks;		// # of member disks
	BYTE gvsRaidLevel; 			// Raid Level of this volume
	BYTE gvsNewMemberDisks;		// Used for migration
	BYTE gvsNewRaidLevel;		// Used for migration
	BYTE gvsRaidSetNumber;		// Raid set number that the volumeset belongs to
	BYTE gvsRes0;				// reserved
	BYTE gvsRes1[4];			// reserved
} sGUI_VOLUMESET, *pGUI_VOLUMESET;


//=======================================================
// Definitions of device(disk) status
//=======================================================
#define  DEV_INITIALIZED 	0x80  /* If this device initialized */
#define  DEV_AVAILABLE 		0x40  /* Available for use */
#define  DEV_HOT_SPARE 		0x20  /* This device is reserved for HOT SPARE */
#define  DEV_FAILED  		0x10  /* Device already failed */
#define  DEV_ATTACHED 		0x08  /* Device attached */
#define  DEV_PASSTHROUGH 	0x02  /* Pass through device */

typedef struct sGUI_PHY_DRV
{
	BYTE gpdModelName[40];			// Drive model name
	BYTE gpdSerialNumber[20];		// Drive serial number
	BYTE gpdFirmRev[8];				// Drive firmware version
	DWORD gpdCapacity;				// Drive capacity in blocks ( A block = 512 bytes )
	DWORD gpdCapacityX; 			// Reserved for expansion
	BYTE gpdDeviceState;			// see above
	BYTE gpdPioMode;
	BYTE gpdCurrentUdmaMode;
	BYTE gpdUdmaMode;
	BYTE gpdDriveSelect;
	BYTE gpdRaidNumber; 			// 0xff if not belongs to a raid set
	sSCSI_ATTR gpdScsi;
	BYTE gpdReserved[40]; 			// Total to 128 bytes
} sGUI_PHY_DRV, *pGUI_PHY_DRV;

/*=======================================================*\
 * Definition for event buffer
\*=======================================================*/
// Event category
enum
{
	EVENT_NO_EVENT,
	EVENT_RAIDSET,
	EVENT_VOLUMESET,
	EVENT_DEVICE,
	EVENT_HOST,
	EVENT_HW_MONITOR,
	EVENT_NEW_83782D
};
// Event type
// 1. RaidSet event 
enum
{
	RS_EVT_CREATE,
	RS_EVT_DELETE,
	RS_EVT_EXPAND,
	RS_EVT_REBUILD,
	RS_EVT_DEGRADED,
	RS_EVT_NO_EVENT
};
// 2. volumeset event
enum
{
	VS_EVT_INITIALIZING,
	VS_EVT_REBUILDING,
	VS_EVT_MIGRATING,
	VS_EVT_CHECKING,
	VS_EVT_COMPLETE_INIT,
	VS_EVT_COMPLETE_REBUILD,
	VS_EVT_COMPLETE_MIGRATING,
	VS_EVT_COMPLETE_CHECKING,
	VS_EVT_CREATE,
	VS_EVT_DELETE,
	VS_EVT_MODIFY,
	VS_EVT_DEGRADED,
	VS_EVT_FAILED,
	VS_EVT_REVIVED
};

#define REBUILD_TYPE_INIT  VS_EVT_INITIALIZING
#define REBUILD_TYPE_REGEN  VS_EVT_REBUILDING
#define REBUILD_TYPE_MIG  VS_EVT_MIGRATING
#define REBUILD_TYPE_CHECK  VS_EVT_COMPLETE_INIT
// IDE device event
enum
{
	DEV_EVT_ADDED,
	DEV_EVT_REMOVED,
	DEV_EVT_READ_ERROR,
	DEV_EVT_WRITE_ERROR,
	DEV_EVT_ATA_ECC_ERROR,
	DEV_EVT_ATA_CHANGE_MODE,
	DEV_EVT_TIMEOUT,
	DEV_EVT_MARK_FAILED,
	DEV_EVT_PCI_ERROR,
	DEV_EVT_SMART_FAILED,
	DEV_EVT_CREATE_PASS,
	DEV_EVT_MODIFY_PASS,
	DEV_EVT_DELETE_PASS
};

// SCSI bus event
enum
{
	SCSI_EVT_RESET,
	SCSI_EVT_PARITY,
	SCSI_BUS_MODE_CHANGE
};

// Hw monitor event
enum
{
	HW_EVT_SDRAM_1BIT_ECC,
	HW_EVT_SDRAM_MULTI_BIT_ECC,
	HW_EVT_TEMP_CONTROLLER,
	HW_EVT_TEMP_BACKPLANE,
	HW_EVT_VOLTAGE15,
	HW_EVT_VOLTAGE3,
	HW_EVT_VOLTAGE5,
	HW_EVT_VOLTAGE12,
	HW_EVT_VOLTAGE1_3,
	HW_EVT_VOLTAGE2_5,
	HW_EVT_VOLTAGE1_25,
	HW_EVT_POWER1_FAILED,
	HW_EVT_FAN1_FAILED,
	HW_EVT_POWER2_FAILED,
	HW_EVT_FAN2_FAILED,
	HW_EVT_POWER3_FAILED,
	HW_EVT_FAN3_FAILED,
	HW_EVT_POWER4_FAILED,
	HW_EVT_FAN4_FAILED,
	HW_EVT_UPS_POWER_LOSS,
	// Add to log H/W monitor recovered
	HW_EVT_TEMP_CONTROLLER_R,
	HW_EVT_TEMP_BACKPLANE_R,
	HW_EVT_VOLTAGE15_R,
	HW_EVT_VOLTAGE3_R,
	HW_EVT_VOLTAGE5_R,
	HW_EVT_VOLTAGE12_R,
	HW_EVT_VOLTAGE1_3_R,
	HW_EVT_VOLTAGE2_5_R,
	HW_EVT_VOLTAGE1_25_R,
	HW_EVT_POWER1_RECOVERED,
	HW_EVT_FAN1_RECOVERED,
	HW_EVT_POWER2_RECOVERED,
	HW_EVT_FAN2_RECOVERED,
	HW_EVT_POWER3_RECOVERED,
	HW_EVT_FAN3_RECOVERED,
	HW_EVT_POWER4_RECOVERED,
	HW_EVT_FAN4_RECOVERED,
	HW_EVT_UPS_POWER_RECOVERED, 
	HW_EVT_SYSTEM_RESTARTED,
	HW_EVT_TEST
};


#if defined(__BIG_ENDIAN__)
typedef union sSYS_TIME
{
	struct
	{
		DWORD tmFlag:1;
		DWORD tmMonth:4;
		DWORD tmYear:5;
		DWORD tmDate:5;
		DWORD tmHour:5;
		DWORD tmMinute:6;
		DWORD tmSecond:6;
	} u;
	struct
	{
		int32_t tmFlag:1;
		int32_t tmTick:31;		
	} x;
} sSYS_TIME, *pSYS_TIME;	

#else
typedef union sSYS_TIME
{
	struct
	{
		DWORD tmSecond:6;
		DWORD tmMinute:6;
		DWORD tmHour:5;
		DWORD tmDate:5;
		DWORD tmYear:5;
		DWORD tmMonth:4;
		DWORD tmFlag:1;
	} u;
	struct
	{
		int32_t tmTick:31;
		int32_t tmFlag:1;
	} x;
} sSYS_TIME, *pSYS_TIME;
// If tmFlag == 1 ==> use YY/MM/DD/HH/MM/SS else use time tick
#endif

/* event object data structure */
typedef struct sEVET_DATA
{
	BYTE evtCategory;			// Event category
	BYTE evtType;				// Event type
	BYTE evtChannel;			// Event channel
	BYTE evtFlag;				// Addition data flag
	int32_t evtTime;			// Occurred time
	DWORD evtData1;				// Additional data1
	DWORD evtData2;				// Additional data2
	BYTE evtStr[16];			// string for event
} sEVENT_DATA, *pEVENT_DATA;	// total 32 bytes


/* strings for event category */
static char *eventCat[] =
{
	"NO_EVENT",
	"RAIDSET",
	"VOLUMESET",
	"DEVICE",
	"HOST",
	"H/W MONITOR"
};
 
/* strings for device event */
static char *htmDevEvent[] =
{
	"Device Inserted",
	"Device Removed",
	"Reading Error",
	"Writing Error",
	"ATA Ecc Error",
	"Change ATA Mode",
	"Time Out Error",
	"Device Failed",
	"PCI Parity Error",
	"Dev Fail (SMART)",
	"PassThrough Disk Created",
	"PassThrough Disk Modified",
	"PassThrough Disk Deleted"
};

/* strings for volumeset event */
static char *htmVolEvent[] =
{
	"Start Initialize",
	"Start Rebuilding",
	"Start Migrating",
	"Start Checking",
	"Complete Init",
	"Complete Rebuild",
	"Complete Migrate",
	"Complete Check",
	"Create Volume",
	"Delete Volume",
	"Modify Volume",
	"Volume Degraded",
	"Volume Failed",
	"Failed Volume Revived"
};

/* strings for raidSet event */
static char *htmRaidEvent[] =
{
	"Create RaidSet",
	"Delete RaidSet",
	"Expand RaidSet",
	"Rebuild RaidSet",
	"RaidSet Degraded"  
};

/* strings for scsi host event */
static char *htmScsiHostEvent[] =
{
	"SCSI Bus Reset",
	"SCSI Bus Parity",
	"SCSI Bus SE<>LVD"
};

/* strings for ide host event */
static char *htmIdeHostEvent[] =
{
	"IDE Bus Reset",
	"IDE Bus UDMA CRC",
	"SCSI Bus SE<>LVD"
};

/* strings for hardware monitor event */
static char *htmHwMonEvent[] =
{
	"DRAM 1-Bit ECC",					
	"DRAM Fatal Error",					
	"Controller Over Temperature",		
	"Backplane Over Temperature",		
	"1.5V Abnormal",
	"3.3V Abnormal",
	"5V Abnormal",
	"12V Abnormal",
	"1.3V Abnormal",
	"2.5V Abnormal",
	"1.25V Abnormal",
	"Power#1 Failed  ",
	"Fan#1 Failed    ",
	"Power#2 Failed  ",
	"Fan#2 Failed    ",
	"Power#3 Failed  ",
	"Fan#3 Failed    ",
	"Power#4 Failed  ",
	"Fan#4 Failed    ",
	"UPS ACPower Loss",  
	"Controller Over Temperature Recovered",
	"Backplane Over Temperature Recovered",
	"1.5V Abnormal Recovered",
	"3.3V Abnormal Recovered",
	"5V Abnormal Recovered",
	"12V Abnormal Recovered",
	"1.3V Abnormal Recovered",
	"2.5V Abnormal Recovered",
	"1.25V Abnormal Recovered",
	"Power#1 Failed Recovered",
	"Fan#1 Failed Recovered",
	"Power#2 Failed Recovered",
	"Fan#2 Failed Recovered",
	"Power#3 Failed Recovered",
	"Fan#3 Failed Recovered",
	"Power#4 Failed Recovered",
	"Fan#4 Failed Recovered",
	"UPS ACPower Loss Recovered",
	"Raid Powered On",
	"Test Event"
};

static char *htmNewEventStr[] =
{
	"Recovered",
	"Over Temp.",
	"Failed",
	"Under Voltage",
	"Over Voltage",
	0,0,0,
	"HTTP Log In",
	"Telnet Log In",
	"VT100 Log In",
	"API Log In",
	"FC Link Up",
	"Fc Link Down"
};


#define UPS_AC_POWER_LOSS	0x03
#define	UPS_OK				0x07

///////////////////////////////////////////////////////////////////////////////////
//
// HWMON_ITEMS
// Indicating the ArcGetHWMon(...) how to retrieve hardware information.
//
#define HWMON_FAN_INFO	0x01
#define	HWMON_VOL_INFO	0x02
#define	HWMON_TMP_INFO	0x03
#define HWMON_PWR_INFO	0x04
#define HWMON_UPS_INFO	0x05

typedef struct _HWMON_INFO
{
	WORD	HwItemCount;	// item count of hardware( including fans, voltages, temperatures, powers)
	WORD	ItemInfo[50];	// item information
}sHwMonInfo, *pHwMonInfo;
///////////////////////////////////////////////////////////////////////////////////
//
//	Hardware monitor information for raid controller excepts ARC-5010 & ARC-6010.
//  ARC-5010 & ARC-6010 should use the "sHwMon5010" data structure instead.
//	Because the number of fans, voltages, temperatures and powers are differ from 
//	models of areca except the ARC-5010 & ARC-6010, so we use the flexible data 
//	structure instead for different models of areca.
//
//  Pass the flag above & pointer of the struct _HWMON_INFO to the ArcGetHWMon(...) to 
//  retrieve the hardware information you want.
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//  For example: Retrieving hardware monitor of fans
//
//	CArclib al;
//  sHwMonInfo	HwInfo;
//	WinCommInterface *iface = new WinCommInterface();
//	if(!iface->init(0))	// init "COM1"
//		return;
//	al.ArcInitSession(iface);
//  stat = al.ArcGetHWMon(HWMON_FAN_INFO,  &HwInfo);
//  
//  if the stat = ARC_SUCCESS, then the contents of HwInfo are:
//  HwInfo.HwItemCount ==> # of fans
//  HwInfo.ItemInfo[0] ==> RPM of 1st fan
//	HwInfo.ItemInfo[1] ==> RPM of 2nd fan
//   ...
//  HwInfo.ItemInfo[HwInfo.HwItemCount - 1] ==> RPM of the last fan
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//	For example: Retrieving hardware monitor of voltage	
//	stat = al.ArcGetHWMon(HWMON_VOL_INFO,  &HwInfo);
//	if the stat = ARC_SUCCESS, then the contents of HwInfo are:
//	HwInfo.HwItemCount ==> # of voltage items
//	HwInfo.ItemInfo[0] ==> base value of 1st voltage
//	HwInfo.ItemInfo[1] ==> current value of 1st voltage
//	HwInfo.ItemInfo[2] ==> base value of 2nd voltage
//	HwInfo.ItemInfo[3] ==> current value of 2nd voltage
//	...
//	HwInfo.ItemInfo[HwInfo.HwItemCount - 2] ==> base value of last voltage
//	HwInfo.ItemInfo[HwInfo.HwItemCount - 1] ==> current value of last voltage
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//	For example: Retrieving hardware monitor of temperature	
//	stat = al.ArcGetHWMon(HWMON_TMP_INFO,  &HwInfo);
//	if the stat = ARC_SUCCESS, then the contents of HwInfo are:
//	HwInfo.HwItemCount ==> # of temperature items
//	HwInfo.ItemInfo[0] ==> controller temperature
//	HwInfo.ItemInfo[1] ==> 1st HDD temperature
//	HwInfo.ItemInfo[2] ==> 2nd HDD temperature
//	...
//	HwInfo.ItemInfo[HwInfo.HwItemCount - 1] ==> last temperature of HDD
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//	For example: Retrieving hardware monitor of power	
//	stat = al.ArcGetHWMon(HWMON_PWR_INFO,  &HwInfo);
//	if the stat = ARC_SUCCESS, then the contents of HwInfo are:
//	HwInfo.HwItemCount ==> # of power items
//	HwInfo.ItemInfo[0] ==> 1st power state
//	HwInfo.ItemInfo[1] ==> 2nd power state
//	...
//	HwInfo.ItemInfo[HwInfo.HwItemCount - 1] ==> last power state
//
//////////////////////////////////////////////////////////////////////////////////////////////////
//  Note: In areca specifications
//	For fans:  the fan speed != 0 means fan ok else failed. 
//	For voltages: one voltage has pair values, one for base value(fixed), the other for current 
//				  voltage value		   
//				  if ( current value > base value * 1.1) or (current value < base value * 0.9)
//				  then the voltage is abnormal.
//  For temperature: Including Controller & HDD temperature, the 1st value is controller temperature 
//					 and the others are HDD temperatures.
//					 If the value over 60 degrees of centigrade, then the temperature is abnormal.
//  For power:	The value is "1" means the power Ok , "0" means failed.
//


///////////////////////////////////////////////////////////////////////////////////
//
//	Hardware monitor data structure for 5 bays machines only (ARC-5010 & ARC-6010).
//	Since the number of fans, voltages, temperatures and powers of the ARC-5010 & ARC-6010 
//  are fixed, so we can use the "fixed" data structure to retrieve hardware information.
//
typedef struct _HWMON_5010
{	
	BYTE	fans;			// # of fans
	BYTE	voltages;		// # of voltages
	BYTE	temps;			// # of temperatures ( only Controller & BackPlane )
	BYTE	powers;			// # of powers
	WORD	monFan;			// fan speed(in RPM)
	WORD	fixed12V;		// always 12000mV(12V)
	WORD	mon12V;			// current voltage of 12000mV
	WORD	fixed5V;		// always 5000mV(5V)
	WORD	mon5V;			// current voltage of 5000mV
	WORD	fixed3_3V;		// always 3300mV(3.3V)
	WORD	mon3_3V;		// current voltage of 3300mV
	WORD	fixed1_5V;		// always 1500mV(1.5V)
	WORD	mon1_5V;		// current voltage of 1500mV
	BYTE	monCtrlTemp;	// current temperature of contoller(in degrees centigrade)
	BYTE	monBkPlaneTemp;	// current temperature of backplane(in degrees centigrade)
	BYTE	reserved[2];	// not use
} sHwMon5010, *pHwMon5010;



///////////////////////////////////////////////////////////////////////////////////
//
//	Volumeset capabilities pass into ArcCreateVolSet(...)
//
typedef struct _VOLUMESET_CAP
{
	DWORD	structs_size;	// size of _VOLUMESET_CAP structure
	BYTE	raid;			// RAIDSet Number that volumeset belongs to.
	char	volname[16];	// volume Set Name
	double	capacity;		// volume capacity size ( in GBs )	
	BYTE	raidlevel;		// raid level
	BYTE	stripsize;		// stripe size (0/1/2/3/4/5->4/8/16/32/64/128K)
	BYTE	channel;		// channel#
	BYTE	id;				// ID#
	BYTE	lun;			// LUN#
	BOOL	tag;			// Enable tag queue(1 : enabled)
	BOOL	cache;			// Enable cache(1 : enabled)
	BYTE	speed;			// speed (0/1/2/3/4->async/20/40/80/160 for scsi)
							// (0/1/2/3/4->33/66/100/133/150 for IDE)
	BYTE	quickinit;		//0:background init, 1:foreground init
}sVolumeSetCap, *pVolumeSetCap;		

///////////////////////////////////////////////////////////////////////////////////
//
//	New volumeset capabilities pass into ArcModifyVolSet(...)
//
typedef struct _NEW_VOLUMESET_CAP
{
	DWORD	structs_size;		// size of _NEW_VOLUMESET_CAP structure
	BYTE	vol;				// VolumeSet Number
	char	newVolname[16];		// volume Set Name
	double	newCapacity;		// volume capacity size in GBs (reserved)
	BYTE	newRaidlevel;		// raid level
	BYTE	newStripsize;		// stripe size (0/1/2/3/4/5->4/8/16/32/64/128K)
	BYTE	newChannel;			// channel#
	BYTE	newId;				// ID#
	BYTE	newLun;				// LUN#
	BOOL	newTag;				// Enable tag queue(1 : enabled)
	BOOL	newCache;			// Enable cache(1 : enabled)
	BYTE	newSpeed;			// speed (0/1/2/3/4->async/20/40/80/160 for scsi)
								// (0/1/2/3/4->33/66/100/133/150 for IDE)
}sNewVolumeSetCap, *pNewVolumeSetCap;		

///////////////////////////////////////////////////////////////////////////////////
//
//	Pass-through disk capabilities pass into ArcCreatePassthroughDisk(...)
//

#define ATA33			0
#define ATA66			1
#define ATA100			2
#define ATA133			3
#define ATA150			4

typedef struct _PASS_DISK_CAP
{
	DWORD	structs_size;	// size of _PASS_DISK_CAP structure
	BYTE	nDevNum;		// device #
	BYTE	nScsiCh;		// scsi channel (0/1)
	BYTE	nScsiId;		// scsi id (0-->15)
	BYTE	nScsiLun;		// scsi lun (0-->7)
	BYTE	nTaggedQue;		// tagged queue (1 : enabled)
	BYTE	nCache;			// cache mode (1 : enabled)
	BYTE	nMaxSpeed;		// (0/1/2/3/4, async/20/40/80/160 for scsi)
							// (0/1/2/3/4, 33/66/100/133/150 for ide)
}sPassDiskCap, *pPassDiskCap;


typedef struct _VOL_MOD
{
	BYTE	num_vol;		// volumeset# to modify 
	BYTE	newlevel;		// new raid level of volumeset
	BYTE	newstripesize;  // new stripe size of volumeset
	BYTE	reserved;		// reserved
}sVolMod, *pVolMod;

typedef struct _VOL_MOD_LIST
{
	BYTE	total_vol;		// specify how many volumesets to modify
	sVolMod	vol_list[32];	// 
}sVolModList, *pVolModList;


#ifndef		OID
#define		OID		long
#endif

///////////////////////////////////////////////////////////////////////////////////
//
//	CArclib definitions
//	This class is exported from the Areca GUI library
// 
#ifdef _WIN32
class ARCLIB_API CArclib 
{
#else
class CArclib 
{		
#endif
// public constructors & destructors
public:
	CArclib();
	~CArclib();

// public member functions for raidsubsystem
public:
	/************************************************************************************
	Routine Name:	ArcGetLibVersion
	Function:		Get the areca library version
	Input:			None.
	Output:			None.
	Return Values:	1. Library version
	Note:			None.
	************************************************************************************/
	int ArcGetLibVersion
	(
		void
	);

	virtual ArcInterface *ArcGetInterface();
	/************************************************************************************
	Routine Name:	ArcInitSession
	Function:		Startup areca library by assigning an interface to it
	Input:			1. ArcInterface
	Output:			None.
	Return Values:	1. ARC_STATUS					
	Note:			This function should be called before any member functions be called
	************************************************************************************/	
	virtual ARC_STATUS ArcInitSession
	(
		IN ArcInterface *Interface
	);

	/************************************************************************************
	Routine Name:	ArcGetSysInfo
	Function:		Get the RAID system information
	Input:			None.
	Output:			1. Pointer to a sSYSTEM_INFO structure that receives the information.					   
	Return Values:	1. ARC_STATUS
	Note:			1. The memory allocation of the sSYSTEM_INFO structure is the 
	                   responsibility of the calling function.		
	************************************************************************************/
	virtual ARC_STATUS ArcGetSysInfo
	(
		OUT pSYSTEM_INFO  pSysInfo
	);

	/************************************************************************************
	Routine Name:	ArcGetRaidSetInfo
	Function:		Get the RaidSet information
	Input:			1. RaidSet number 					
	Output:			1. Pointer to a sGUI_RAIDSET structure that receives the information.
	Return Values:	1. ARC_STATUS
	Note:			1. RaidSet number is started from 0 to sSYSTEM_INFO.gsiMaxRaidSet
	                2. The memory allocation of the sGUI_RAIDSET structure is the 
	                   responsibility of the calling function.
	************************************************************************************/
	virtual ARC_STATUS ArcGetRaidSetInfo
	(
		IN	char iRaidNumber, 
		OUT pGUI_RAIDSET pRaidInfo
	);

	/************************************************************************************
	Routine Name:	ArcGetVolSetInfo
	Function:		Get the VolumeSet information
	Input:			1. VolumeSet number					
	Output:			1. Pointer to a sGUI_VOLUMESET structure that receives the information.
	Return Values:	1. ARC_STATUS
	Note:			1. VolumeSet number is started from 0 to sSYSTEM_INFO.gsiMaxVolumeSet
	                2. The memory allocation of the sGUI_VOLUMESET structure is the 
	                   responsibility of the calling function.
	************************************************************************************/
	virtual ARC_STATUS ArcGetVolSetInfo
	(
		IN	char iVolNumber, 
		OUT pGUI_VOLUMESET pVolSet
	);

	/************************************************************************************
	Routine Name:	ArcGetPhysicalDriveInfo
	Function:		Get the physical drive information
	Input:			1. Physical dirve number					
	Output:			1. Pointer to a sGUI_PHY_DRV structure that receives the information.
	Return Values:	1. ARC_STATUS
	Note:			1. Drive number is started from 0 to sSYSTEM_INFO.gsiIdeChannels
	                2. The memory allocation of the sGUI_PHY_DRV structure is the 
	                   responsibility of the calling function.
	************************************************************************************/
	virtual ARC_STATUS ArcGetPhysicalDriveInfo
	(
		IN	char iDriveNumber, 
		OUT	pGUI_PHY_DRV pPhyDrv
	);

	/************************************************************************************
	Routine Name:	ArcGetHWMon
	Function:		Get the H/W monitor status
	Input:			1. Hardware monitor item 
	Output:			1. Pointer to a sHwMonInfo structure that receives the information.
	Return Values:	1. ARC_STATUS
	Note:			1. HwItem is described by HWMON_ITEMS
	                2. The memory allocation of the sHwMonInfo structure is the 
	                   responsibility of the calling function.
	************************************************************************************/
	virtual ARC_STATUS ArcGetHWMon
	( 
		IN	DWORD HwItem, 
		OUT	pHwMonInfo pHwInfo 
	);

	/************************************************************************************
	Routine Name:	ArcGetHWMon5010
	Function:		Get the H/W monitor status for 5010 & 6010 controller only
	Input:			None.
	Output:			1. Pointer to a sHwMon5010 structure that receives the information.
	Return Values:	1. ARC_STATUS
	Note:			1. The memory allocation of the sHwMon5010 structure is the 
	                   responsibility of the calling function.
	************************************************************************************/
	virtual ARC_STATUS ArcGetHWMon5010
	( 
		OUT pHwMon5010 pHwMon 
	);

	/************************************************************************************
	Routine Name:	ArcSetSerial
	Function:		Set serial number of RAID subsystem
	Input:			1. Serial number(null terminate), Max 16 characters
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcSetSerial
	(
		IN char *szSerialNumber
	);

	/************************************************************************************
	Routine Name:	ArcSetVendor
	Function:		Set vendor string of RAID subsystem
	Input:			1. Vendor string(null-terminated), Max 40 characters
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcSetVendor
	(
		IN char *szVerdorName
	);

	/************************************************************************************
	Routine Name:	ArcSetModel
	Function:		Set model string of RAID subsystem
	Input:			1. Model string(null-terminated), Max 8 characters
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcSetModel
	(
		IN char *szModelName
	);

	/************************************************************************************
	Routine Name:	ArcGetIdentifyString
	Function:		Get controller identify string
	Input:			None.
	Output:			1. Pointer to a buffer that receives the information.
	Return Values:	1. ARC_STATUS
	Note:			1. The memory allocation of the buffer is the responsibility of the 
	                   calling function and the buffer must be >= 44 bytes 
	************************************************************************************/
	virtual ARC_STATUS ArcGetIdentifyString
	(
		OUT char *strBuf
	);

	/************************************************************************************
	Routine Name:	ARC_STATUS ArcIdentify(void)
	Function:		Identify controller
	Input:			None
	Output:			None
	Return Values:	1. ARC_STATUS
	Note:			1. If ARC_STATUS == ARC_SUCCESS then the controller found else
	                   no controller found.
	************************************************************************************/
	virtual ARC_STATUS ArcIdentify
	(
		void
	);

	/************************************************************************************
	Routine Name:	ArcMuteBeeper
	Function:		Mute the beeper
	Input:			None
	Output:			None
	Return Values:	1. ARC_STATUS 				
	Note:			1. Password required if password enabled, use ArcChkPassword(...)
					   to validate the password.						
	************************************************************************************/
	virtual ARC_STATUS ArcMuteBeeper
	( 
		void 
	);

	/************************************************************************************
	Routine Name:	ArcSetBeeper
	Function:		Set the Beeper state
	Input:			1. DISABLE or ENABLE
	Output:			None
	Return Values:	1. ARC_STATUS  				
	Note:			1. Password required if password enabled, use ArcChkPassword(...)
					   to validate the password.
	************************************************************************************/
	virtual ARC_STATUS ArcSetBeeper
	(
		IN BYTE bSetting
	);

	/************************************************************************************
	Routine Name:	ArcSetMacAddr
	Function:		Set MAC Address of RAID subsystem if ethernet supported
	Input:			1. Standard MAC Address format, must be 6 byte characters.
	Output:			None
	Return Values:	1. ARC_STATUS
	Note:			1. Example of MAC Address format, 
	                   char mac[6] = { 0x00, 0x04, 0xD9, 0x60, 0x00, 0x00 };
	************************************************************************************/
	virtual ARC_STATUS ArcSetMacAddr
	(
		IN char *MacAddress
	);

	/************************************************************************************
	Routine Name:	ArcSetLogo
	Function:		Set the OEM logo
	Input:			1. The buffer contains the context of the image( .jpg or .gif )
					2. The actually size of image, not the buffer size.
	Output:			None
	Return Values:	1. ARC_STATUS
	Note:			1. Image size must be <= 7998 bytes
					2. The sequences of set logo:
					   2a. Read the image file from disk into the buffer that calling
					       function allocated.
					   2b. Pass the buffer and image size as the parameter into the 
					       member function ArcSetLogo(...)
	************************************************************************************/
	virtual ARC_STATUS ArcSetLogo
	(
		IN char *imageBuf, 
		IN int imageSize
	);

	/************************************************************************************
	Routine Name:	ArcSetTime
	Function:		The ArcSetTime method synchronizes the RAID subsystem data-time with 
	                the host date-time.
	Input:			None
	Output:			None
	Return Values:	1. ARC_STATUS
	Note:			None
	************************************************************************************/
	virtual ARC_STATUS ArcSetTime
	(
		void 
	);

	/************************************************************************************
	Routine Name:	ArcSNMP
	Function:		Query SNMP value with SNMP standard message ( BER encoding )
	Input:			1. SNMP Message ( BER encoding )
					2. Length of SNMP Message
	Output:			1. SNMP Message returned from RAID subsystem ( BER encoding )
					2. The length of returning SNMP message
	Return Values:	1. ARC_STATUS
	Note:			1. This member function is available if sSYSTEM_INFO.gsiRs232Snmp were true.
	************************************************************************************/
	virtual ARC_STATUS ArcSNMP
	(
		IN	char *SNMPMsg, 
		IN	int length, 
		OUT	char *SNMPResponse, 
		OUT	int *ResLength
	);

	/************************************************************************************
	Routine Name:	ArcSNMPWithOid
	Function:		Query SNMP value with OID name
	Input:			1. The OID from the NMS Request
					2. Length of OID( both OID-IN and OID-OUT )
					3. Community string
					4. Exact ( if exact == 1 then SNMPGET else SNMPGETNEXT  )
	Output:			1. The OID returned from RAID-Subsystem 
					2. SNMP tag of the returning value
					3. The answer of this OID
					4. The length of this answer
	Return Values:	1. ARC_STATUS
	Note:			1. This member function is available if sSYSTEM_INFO.gsiRs232Snmp were true.
	                2. That length serve as a dual purpose in both IN and OUT.
					3. The calling thread has the responsibility of freeing memory of var
					   which allocates by ArcSNMPWithOid(...)
	************************************************************************************/
	virtual ARC_STATUS ArcSNMPWithOid
	(
		IN	OID	*oid, 
		IN OUT int	*length, 
		IN  unsigned char *szCommunity,
		OUT OID	*ret_oid, 
		OUT	unsigned char *tag, 
		OUT	unsigned char **var,
		OUT	int		*var_len,
		IN unsigned char exact = 0 // default SNMPGETNEXT
	);

	/************************************************************************************
	Routine Name:	ArcSetPassword
	Function:		Set the password of the RAID Subsystem
	Input:			1. New password with null-terminated
	Output:			None.  
	Return Values:	1. ARC_STATUS
	Note:			1. New password string must be <= 15 characters	
	************************************************************************************/
	virtual ARC_STATUS ArcSetPassword
	(
		IN char *szNewPasswordString
	);  

	/************************************************************************************
	Routine Name:	ArcSetHostMode
	Function:		Set host mode
	Input:			0->Independent(RAID), 1->cluster(JBOD)
	Output:			None.  	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcSetHostMode
	(
		IN BYTE bHostMode
	);

	/************************************************************************************
	Routine Name:	ArcSetRebuildPriority
	Function:		Rebuild priority setting
	Input:			0/1/2/3 (low->high)
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcSetRebuildPriority
	(
		IN BYTE bPriority
	);

	/************************************************************************************
	Routine Name:	ArcSetMaxAtaMode
	Function:		Set max ATA mode
	Input:			0/1/2/3 (133/100/66/33)	                
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcSetMaxAtaMode
	(
		IN BYTE bAtaMode
	);

	/************************************************************************************
	Routine Name:	ArcSetDhcpIp
	Function:		Set DHCP and IP
	Input:			1. 0:dhcp disabled, 1:dhcp enabled
					2. Null-terminated character string representing a number expressed in the Internet standard "."
					   (dotted) notation, format: "xxx.xxx.xxx.xxx" ex: 192.168.001.100
	Output:			None.  		
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcSetDhcpIp
	(
		IN BYTE bDhcp, 
		IN PBYTE szIp
	);

	/************************************************************************************
	Routine Name:	ArcSetRaidTermPort
	Function:		Set Raid Terminal Port
	Input:			1. 0->COMA (term port), 1->COMB (debug port)
					2. 0/1/2/3/4/5/6/7 (1200/2400/4800/9600/19200/38400/57600/115200)
					3. stop bit (0:1, 1:2 stop bits)
	Output:			None.  
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcSetTermPort
	(
		IN BYTE nPort, 
		IN BYTE nBaudRate, 
		IN BYTE nStopBit
	);

	/************************************************************************************
	Routine Name:	ArcChkPassword
	Function:		Pass the password that user input to RAID Subsystem
	Input:			1. Password with null-terminated
	Output:			None.	
	Return Values:	1. ARC_STATUS
	Note:			1. Password string must be <= 15 characters	
	************************************************************************************/
	virtual ARC_STATUS ArcChkPassword
	(
		IN char *szPassword
	);

	/************************************************************************************
	Routine Name:	ArcLogOut
	Function:		Logout GUI (force password checking on next command)
	Input:			None.
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcLogOut
	( 
		void 
	);

	/************************************************************************************
	Routine Name:	ArcHttp
	Function:		Translates the HTTP data between BROWSER and RAID Subsystem
	Input:			1. HTTP data from browser( HTML format )
					2. The length of the HTTP data from browser
	Output:			1. The HTTP data returned from RAID subsystem ( HTML format)
					2. The Length of the returning HTTP data
	Return Values:	1. ARC_STATUS
	Note:			1. Sequence of translation.
					   1a. Setup the sockets for waiting connection from remote side.
					   1b. Receive the HTTP data from remote side.
					   1c. Pass the received data and length into ArcHttp.
					   1d. Send the HTTP data returned from RAID subsystem back to remote side.
	************************************************************************************/
	virtual ARC_STATUS ArcHttp
	(
		IN	char *pClientHttpData, 
		IN	int nClientHttpDataSize, 
		OUT	char *pRaidReturnHttp, 
		OUT	int *nRaidReturnSize
	);

	/************************************************************************************
	Routine Name:	ArcPollEvent
	Function:		Get the currently index of event object in the RAID subsystem
	Input:			None.
	Output:			1. Pointer to an INTEGER to save the index of current event object
	Return Values:	1. ARC_STATUS
	Note:			1. There are 64 event objects inside the RAID subsystem
	************************************************************************************/
	virtual ARC_STATUS ArcPollEvent
	(
		OUT int *nCurrentEvtIndex
	);

	/************************************************************************************
	Routine Name:	ArcGetReqEventPage
	Function:		Get the specific number of event pages from RAID subsystem
	Input:			1. Indicate how to retrieve the event object(s), this parameter can 
	                   be one of the following values:
					   1a. FLAG_ALL, retrieve all event objects inside the RAID subsystem
					   1b. FLAG_NEW, retrieve only new event objects
	Output:			1. How many event objects we retrieved
	Return Values:	1. ARC_STATUS	
	Note:			None.
	************************************************************************************/
	virtual ARC_STATUS ArcGetReqEventPage
	(
		IN	char bFlag, 	
		OUT int *iCount
	);

	/************************************************************************************
	Routine Name:	ArcGetEventObject
	Function:		Get the individual event object
	Input:			1. Event object index 
	Output:			1. Pointer to an sEVENT_DATA to save the current event object
	Return Values:	1. boolean value, > 0 means successed
	Note:			1. 
	************************************************************************************/
	virtual BOOL ArcGetEventObject
	(
		IN	int nEvtNum, 
		OUT	pEVENT_DATA pEventObject
	);

	/************************************************************************************
	Routine Name:	ArcClearEvent
	Function:		Clear all event buffer in the RAID subsystem
	Input:			None.
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcClearEvent
	( 
		void 
	);

	/************************************************************************************
	Routine Name:	ArcCreateRaidSet
	Function:		Create RaidSet
	Input:			1. Device mask, bit0 -- drive 0, bit1 -- drive 1 ....
					   ex: Device mask = 0x0000000F means that create raidset with device 0,1,2 and 3
					2. RaidSet name ( <= 16 chars with null-terminated )
	Output:			None.	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcCreateRaidSet
	(
		IN DWORD nDevMask, 
		IN char *szRaidName
	);

	/************************************************************************************
	Routine Name:	ArcDelRaidSet
	Function:		Delete the particular RaidSet
	Input:			1. RaidSet number to specify which one to be deleted
	Output:			None.  	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcDelRaidSet
	(
		IN char nRaidNum
	);

	/************************************************************************************
	Routine Name:	ArcExpandRaidSet
	Function:		Expand RaidSet with volumeset(s) modification
	Input:			1. RaidSet number to specify which one to be expanded
					2. Device mask(see ArcCreateRaidSet(...))
					3. The volumeset(s) will be modified 
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcExpandRaidSet
	( 
		IN	char nRaidNum, 
		IN	DWORD nDevMask,
		IN	pVolModList	volmodlist = NULL
	);


	/************************************************************************************
	Routine Name:	ArcActiveRaidSet
	Function:		Activate incomplete raid set
	Input:			1. RaidSet number to specify which one to be activated
	Output:			None.  	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcActiveRaidSet
	(
		IN	char nRaidNum
	);

	/************************************************************************************
	Routine Name:	ArcCreateHotSpare
	Function:		Create Hot Spare Disk
	Input:			1. Device mask( see ArcCreateRaidSet(...))
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcCreateHotSpare
	(
		IN DWORD nDevMask
	);

	/************************************************************************************
	Routine Name:	ArcDeleteHotSpare
	Function:		Delete Hot Spare Disk
	Input:			1. Device mask(see ArcCreateRaidSet(...))
	Output:			None.		
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcDeleteHotSpare
	(
		IN DWORD nDevMask
	);

	/************************************************************************************
	Routine Name:	ArcCreateVolSet
	Function:		Create VolumeSet
	Input:			1. Pointer to a sVolumeSetCap structure that uses to create VolumeSet
	                2. Specify the type to create over 2TB volumeset.
					   TYPE_64BIT_LBA: 64bitLBA(default), TYPE_4K_FOR_WIN: 4KByte for Windows
	Output:			None.  		
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcCreateVolSet
	(
		IN pVolumeSetCap pVol,
		IN BYTE Over2TBType = TYPE_64BIT_LBA
	);

	/************************************************************************************
	Routine Name:	ArcModifyVolSet
	Function:		Modify VolumeSet
	Input:			1. Pointer to a sNewVolumeSetCap structure that uses to modify VolumeSet
	Output:			None.  				
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcModifyVolSet
	(
		IN pNewVolumeSetCap pNewVol
	);

	/************************************************************************************
	Routine Name:	ArcDelVolSet
	Function:		Delete VolumeSet
	Input:			1. VolumeSet number to specify which one to be deleted
	Output:			None.  	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcDelVolSet
	(
		IN char nVolNum
	);

	/************************************************************************************
	Routine Name:	ArcStartCheckVolume
	Function:		Start volume consistency check
	Input:			1. Volume number to specify which one be checked
	Output:			None.  	
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcStartCheckVolume
	(
		IN char nVolNum
	);

	/************************************************************************************
	Routine Name:	ArcStopCheckVolume
	Function:		Stop volume consistency check
	Input:			None.
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcStopCheckVolume
	(
		void
	);

	/************************************************************************************
	Routine Name:	ArcCreatePassthroughDisk
	Function:		Create pass-through disk
	Input:			1. Pointer to a sPassDiskCap structure that uses to create pass-through Disk
	Output:			None.  		
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcCreatePassthroughDisk
	(
		IN pPassDiskCap pDisk
	);

	/************************************************************************************
	Routine Name:	ArcModifyPassthroughDisk
	Function:		Modify pass-through disk
	Input:			1. Pointer to a sPassDiskCap structure that uses to create pass-through Disk
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcModifyPassthroughDisk
	(
		IN pPassDiskCap pNewDiskCap
	);

	/************************************************************************************
	Routine Name:	ArcDelPassthroughDisk
	Function:		Delete the pass-through disk
	Input:			1. Disk number specify which one to be deleted
	Output:			None.  				
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
	************************************************************************************/
	virtual ARC_STATUS ArcDelPassthroughDisk
	(
		IN char nDriveNum
	);


	/************************************************************************************
    Routine Name:	ArcIdentifyDisk
    Function:		Identify the disk
    Input:			1. Device mask(see ArcCreateRaidSet(...))
    Output:			None.  	
    Return Values:	1. ARC_STATUS
    Note:			1. Set the Device mask to zero to stop the disk identifying.
	                2. Password required if password enabled	                
	************************************************************************************/
    virtual ARC_STATUS ArcIdentifyDisk
    (
        IN DWORD nDriveMask
    );

	/************************************************************************************
	Routine Name:	ArcResetController
	Function:		Restart controller
	Input:			None.
	Output:			None.
	Return Values:	1. ARC_STATUS
	Note:			1. Password required if password enabled
					2. After restart controller, it is essential to wait controller to 
					   get ready, the time may be 10~60 seconds depends on the models
	************************************************************************************/
	virtual ARC_STATUS ArcResetController( void );	

protected:
	ARC_STATUS ArcGetEvent(char nPage);
	ARC_STATUS SetLogoPage(BYTE nPage, BYTE *Data, int len);
	BOOL	ArcCkFrameValid(PBYTE frame, DWORD dwLength);
	BYTE	CalCheckSum( PBYTE src, int len);	
	BOOL	IsValidHandle();
	// How the library talk to raidsubsystem
	ArcInterface *m_Interface;

private:
	// save the previous index of event
	int		m_iPreIndex;
	// save the current index of event
	int		m_iCurrentIndex;
	int		m_iNewCount;
	BYTE	*m_pEventPage;	// space for 1 event page
	// memory mapping for event space inside the raid controller.
	// these are 4 event pages inside the raid subsystem, and each contains 512 bytes
	// so total size = 512 * 4 (bytes)
	BYTE	*m_pEventMapping;	
};

#ifdef __cplusplus
}
#endif

#endif	// # _ARCLIB_H

